/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;

import junit.framework.TestCase;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public abstract class DataSerializationTest extends TestCase
{
    public static final int THREADS=5;
    public static final int LOOPS=250;
    public static final int START_MEASURE=100;
    

    
    /** I can't use JBossSerialization for this operation, as it would invalidate the test itself */
	public static Object cloneUsingJavaSerializatoin(Object obj) throws Exception {
		synchronized (obj)
		{
			ByteArrayOutputStream array = new ByteArrayOutputStream();
			ObjectOutputStream cloneOutput = new ObjectOutputStream(array);
			cloneOutput.writeObject(obj);
			cloneOutput.flush();
			
			ObjectInputStream cloneInput = new ObjectInputStream(new ByteArrayInputStream(array.toByteArray()));
			Object newObj =  cloneInput.readObject();
			if (!(obj instanceof Object[]))
			{
				if (!obj.equals(newObj))
				{
					throw new RuntimeException("Object didn't Match during cloning");
				}
			}
			return newObj;
		}
	}

	String fileName;
    
    public DataSerializationTest()
    {
    	super();
    }
    
    public DataSerializationTest(String fileName)
    {
    	super(fileName);
    	if (fileName.endsWith(".class"))
    	{
    		this.fileName=fileName;
    	}
    }

   public void testDataSerialization() throws Throwable
   {
      String packagePrefix = "org.jboss.serial.data";

      String dataFilePath = DataSerializationTest.class.getResource("data").getFile();
      File dataFile = new File(dataFilePath);

      if(!dataFile.exists())
      {
         throw new Exception("Can not run test as data package for data objects can not be found.");
      }

      boolean hasFailures=false;
      beforeTests();
      String[] testFiles = dataFile.list();
      for(int x = 0; x < testFiles.length; x++)
      {
    	 try
    	 {
	         String testFileName = testFiles[x];
	         boolean isFileValid=false;
	         if (fileName!=null)
	         {
	        	 isFileValid=testFileName.equals(fileName);
	         }
	         else
	         {
	        	 isFileValid=testFileName.endsWith(".class") && testFileName.startsWith("Test") && testFileName.lastIndexOf('$')==-1;
	         }
	        	 
	         if(isFileValid)
	         {
	            String fqnTestFile = packagePrefix + "." + testFileName.substring(0, testFileName.length() - 6);
	//System.out.println("Data object class name = " + fqnTestFile);
	            Class testClass = Class.forName(fqnTestFile);
	            Method method = testClass.getMethod("createTestInstance",new Class[]{});
	            Object dataObject = method.invoke(this, new Object[]{});
	//System.out.println("dataObject = " + dataObject);
	            executTest(dataObject);
	         }
    	 }
    	 catch (Throwable e)
    	 {
    		 e.printStackTrace(System.out);
    		 hasFailures=true;
    	 }
      }
      if (hasFailures)
      {
    	  fail("Test had failures during its execution. Look at log for more details");
      }
      afterTests();
   }

   public void beforeTests() throws Throwable
   {
   }

   public void afterTests() throws Throwable
   {
   }

   public abstract void executTest(Object dataObject) throws Throwable;

}