/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.benchmarks;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;

import org.jboss.serial.DataSerializationTest;
import org.jboss.serial.data.NonSerializableTest;
import org.jboss.serial.io.JBossObjectInputStream;
import org.jboss.serial.io.JBossObjectOutputStream;
import org.jboss.serial.io.MarshalledObject;
import org.jboss.serial.io.MarshalledObjectForLocalCalls;
import org.jboss.serial.objectmetamodel.DataContainer;
import org.jboss.serial.objectmetamodel.safecloning.SafeClone;
import org.jboss.serial.objectmetamodel.safecloning.SafeCloningRepository;
import org.jboss.serial.util.StringUtilBuffer;

/**
 * Benchmarks the size of streaming
 * @author Clebert Suconic
 *
 */
public class SizeBenchmarkTestCase extends DataSerializationTest
{
	
	private static final int MAX_LOOP = 1;
	private static final boolean SAVE_FILE=System.getProperties().getProperty("SizeBenchmarkTestCase.saveFiles")!=null;

	static ByteArrayOutputStream byteOut = new ByteArrayOutputStream();

	public SizeBenchmarkTestCase(String file)
	{
		super(file);
	}
	
	public SizeBenchmarkTestCase()
	{
		super();
	}
	
	public void executTest(Object dataObject) throws Throwable
	{
    	if (dataObject instanceof NonSerializableTest)
    	{
    		return;
    	}
		System.out.println("Testing " + dataObject.getClass().getName());
		try
		{
			doTestJavaSerialization(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}

		try
		{
			doTestJBossSerialization(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			//throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
		
		try
		{
			doTestJBossSerializationInDirect(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
		
		try
		{
			doTestJBossMarhsalledValue(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}

		try
		{
			doTestJBossMarhsalledValueOverSerialization(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}

		try
		{
			doTestJBossMarhsalledValueLocal(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}

		try
		{
			doTestJBossMarhsalledValueOverSerializationLocal(dataObject);
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
		
		try
		{
			doTestJBossDataContainer(dataObject);
			
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
		
		try
		{
			doTestJBossDataContainerWithSmartClone(dataObject);
			
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
		
		try
		{
			doTestJBossDataContainerWithSmartClone2(dataObject);
			
		}
		catch (Throwable e)
		{
			e.printStackTrace(System.out);
			throw new Exception("Failed at " + dataObject.getClass().getName(),e);
		}
	}
	
	// do not change these testcases as they test the interface used by JBossRemoting
    private void doTestJBossDataContainer(Object dataObject) throws Exception
    {
        DataContainer container = new DataContainer(false);
        ObjectOutput output = container.getOutput();
        output.writeObject(dataObject);
        output.flush();
        container.flush();
        
        container.getCache().setLoader(Thread.currentThread().getContextClassLoader());
        Object newObject = container.getInput().readObject();
        if (!(dataObject instanceof String) && !(dataObject instanceof String[])) 
        {
        	assertNotSame(dataObject,newObject);
            assertEquals(dataObject,newObject);
        }
        
   }

	// do not change these testcases as they test the interface used by JBossRemoting
    private void doTestJBossDataContainerWithSmartClone(Object dataObject) throws Exception
    {

    	SafeClone safeClone = new SafeClone()
    	{
			public boolean isSafeToReuse(Object obj) {
				return false;
			}
    	};
    	
    	SafeCloningRepository safeToReuse = new SafeCloningRepository(safeClone);
    	
    	DataContainer container = new DataContainer(null, null, safeToReuse, false);
        ObjectOutput output = container.getOutput();
        output.writeObject(dataObject);
        output.flush();
        container.flush();
        
        container.getCache().setLoader(Thread.currentThread().getContextClassLoader());
        Object newObject = container.getInput().readObject();
        if (!(dataObject instanceof String) && !(dataObject instanceof String[]))
        {
        	assertEquals(dataObject,newObject);
            assertNotSame(dataObject,newObject);
        }
   }

	// do not change these testcases as they test the interface used by JBossRemoting
    private void doTestJBossDataContainerWithSmartClone2(Object dataObject) throws Exception
    {
    	JBossObjectOutputStream objInput = new JBossObjectOutputStream(null);
    	Object newObject = objInput.smartClone(dataObject);


    	if (!(dataObject instanceof String) && !(dataObject instanceof String[]))
    	{
	    	assertEquals(dataObject,newObject);
	    	assertNotSame(dataObject,newObject);
    	}
        
   }

	public void doTestJBossSerialization(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
            byteOut.reset();

            JBossObjectOutputStream out = new JBossObjectOutputStream(byteOut,buffer);
            out.writeObject(myTest);
            out.flush();


            byte[] byteArray = byteOut.toByteArray();
            saveFile("/tmp/jboss-test-" + myTest.getClass().getName() + ".bin",byteArray);
            System.out.println(myTest.getClass().getName() +" produced " + byteArray.length + " on JBossSerialization");
            ByteArrayInputStream byteInput = new ByteArrayInputStream(byteArray);

            JBossObjectInputStream input = new JBossObjectInputStream(byteInput,buffer);

            Object value = input.readObject();

            assertNotSame(myTest,value);
            if (!(myTest instanceof String) && !(myTest instanceof String[]))
            {
	            assertSame(myTest.getClass(),value.getClass());
	            assertEquals(myTest,value);
            }
        }


    }

    public void doTestJBossSerializationInDirect(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
            byteOut.reset();

            JBossObjectOutputStream out = new JBossObjectOutputStream(byteOut,buffer);
            out.writeObjectUsingDataContainer(myTest);
            out.flush();


            byte[] byteArray = byteOut.toByteArray();
            saveFile("/tmp/jboss-direct-test-" + myTest.getClass().getName() + ".bin",byteArray);
            System.out.println(myTest.getClass().getName() +" produced " + byteArray.length + " on doTestJBossSerializationInDirect");
            ByteArrayInputStream byteInput = new ByteArrayInputStream(byteArray);

            JBossObjectInputStream input = new JBossObjectInputStream(byteInput,buffer);

            Object value = input.readObjectUsingDataContainer();

            assertNotSame(myTest,value);
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }

    public void doTestJBossMarhsalledValue(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
        	byteOut.reset();
        	MarshalledObject obj = new MarshalledObject(myTest);
            Object value = obj.get();

            assertNotSame(myTest,value);
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }

    public void doTestJBossMarhsalledValueOverSerialization(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
        	MarshalledObject marshall = new MarshalledObject(myTest);
            
        	byteOut.reset();
            ObjectOutputStream regularOutput = new ObjectOutputStream(byteOut);
            regularOutput.writeObject(marshall);
            regularOutput.flush();
            
            ObjectInputStream regularInput = new ObjectInputStream(new ByteArrayInputStream(byteOut.toByteArray()));
            marshall  = (MarshalledObject)regularInput.readObject();

            Object value = marshall.get();
            assertNotSame(myTest,value);
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }

    public void doTestJBossMarhsalledValueLocal(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
        	MarshalledObjectForLocalCalls obj = new MarshalledObjectForLocalCalls(myTest);
            Object value = obj.get();

            if (!(myTest instanceof String))
            {
            	assertNotSame(myTest,value);
            }
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }

    public void doTestJBossMarhsalledValueOverSerializationLocal(Object myTest) throws Exception
    {
        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);

        for (int exec=0;exec<MAX_LOOP;exec++)
        {
        	MarshalledObjectForLocalCalls marshall = new MarshalledObjectForLocalCalls(myTest);
            
        	byteOut.reset();
            ObjectOutputStream regularOutput = new ObjectOutputStream(byteOut);
            regularOutput.writeObject(marshall);
            regularOutput.flush();
            
            ObjectInputStream regularInput = new ObjectInputStream(new ByteArrayInputStream(byteOut.toByteArray()));
            marshall  = (MarshalledObjectForLocalCalls)regularInput.readObject();

            Object value = marshall.get();
            assertNotSame(myTest,value);
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }
    
    private static void saveFile(String fileName, byte [] bytes) throws Exception
    {
    	if (SAVE_FILE)
    	{
	    	saveFileNoCheck(fileName, bytes);
    	}
    }

	public static void saveFileNoCheck(String fileName, byte[] bytes) throws FileNotFoundException, IOException {
		File file = new File(fileName);
		DataOutputStream dataout = new DataOutputStream (new FileOutputStream(file));
		dataout.write(bytes);
		dataout.close();
	}

    public void doTestJavaSerialization(Object myTest) throws Exception
    {
        for (int exec=0;exec<MAX_LOOP;exec++)
        {
            byteOut.reset();

            ObjectOutputStream out = new ObjectOutputStream(byteOut);
            out.writeObject(myTest);
            out.flush();

            byte[] byteArray = byteOut.toByteArray();
            saveFile("/tmp/java-test-" + myTest.getClass().getName() + ".bin",byteArray);
            System.out.println(myTest.getClass().getName() +" produced " + byteArray.length + " on JavaSerialization");
            ByteArrayInputStream byteInput = new ByteArrayInputStream(byteArray);

            ObjectInputStream input = new ObjectInputStream(byteInput);

            Object value = input.readObject();

            assertNotSame(myTest,value);
            assertSame(myTest.getClass(),value.getClass());
            if (!(myTest instanceof String[]))assertEquals(myTest,value);
        }


    }
    
}
