/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.classmetamodel;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.Serializable;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;

import junit.framework.Test;
import junit.framework.TestCase;

import org.jboss.jrunit.decorators.ThreadLocalDecorator;
import org.jboss.serial.DataSerializationTest;
import org.jboss.serial.io.JBossObjectOutputStream;


/**
 * @author clebert suconic
 */
public class ClassMetamodelFactoryTestCase extends TestCase
{

    static class A implements Serializable
    {
        public A()
        {

        }
        private B b;
        long internal;
    }

    static class B implements Serializable
    {
        private C c;
    }

    static class C implements Serializable
    {
        private A a;
    }

    static class NonSerializable
    {
        private A a;
    }

    static class ExternalizableClass implements Externalizable
    {
        private A a;

        public void writeExternal(ObjectOutput out) throws IOException
        {

        }

        public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
        {
        }
    }

    // the meta model will have to extend its calls
    static class D extends A
    {
        public D()
        {
        }

        private int c;
        // not supposed to be in the meta model
        private transient int d;
        // not supposed to be in the meta model
        private static transient int e;
        // not supposed to be in the meta model
        private static int f;

        private int dummy;
        private String[] z;
    }

    public static Test suite() throws Exception
    {
        return new ThreadLocalDecorator(ClassMetamodelFactoryTestCase.class,5,10,0,false)        {
            public void threadsTearDown()
            {
                super.threadsTearDown();
            }
        };

    }


    public void testgetClassMetaData() throws Exception
    {
        ClassMetaData metaData = ClassMetamodelFactory.getClassMetaData(A.class,true);

        ClassMetaData metaDataD = ClassMetamodelFactory.getClassMetaData(D.class,true);

        assertEquals(1,metaData.getSlots().length);
        assertEquals(2,metaDataD.getSlots().length);
        assertEquals(2,metaDataD.getSlots()[0].getFields().length);
        assertEquals(3,metaDataD.getSlots()[1].getFields().length);
    }

    public void testNonSerializable()
    {
        try
        {
            ClassMetaData metaData = ClassMetamodelFactory.getClassMetaData(NonSerializable.class,true);
            fail("Supposed to send an exception");
        }
        catch (IOException e)
        {
        }
    }

    public void testExternalizable()
    {
        try
        {
            ClassMetaData metaData = ClassMetamodelFactory.getClassMetaData(Externalizable.class,true);
        }
        catch (IOException e)
        {
            fail("Not Supposed to send an exception");
        }
    }

    public void testImmutable()
    {
        assertTrue(ClassMetamodelFactory.isImmutable(Long.class));
        assertTrue(ClassMetamodelFactory.isImmutable(Long.TYPE));
    }


    public void testFindClass() throws Exception
    {
        ClassMetamodelFactory.cache.clear();
        ClassMetaData metaData = ClassMetamodelFactory.getClassMetaData(Hashtable.class,true);
        metaData = ClassMetamodelFactory.getClassMetaData(Hashtable.class,true);
        metaData = ClassMetamodelFactory.getClassMetaData(Hashtable.class,true);
        metaData = ClassMetamodelFactory.getClassMetaData(Hashtable.class,true);
        metaData = ClassMetamodelFactory.getClassMetaData(Hashtable.class,true);
    }

    public static void printCache()
    {
        System.out.println("Size of cache = " + ClassMetamodelFactory.cache.size());
        Iterator iter = ClassMetamodelFactory.cache.keySet().iterator();
        while (iter.hasNext())
        {
            System.out.println("Item = " + iter.next());
        }
    }

    /**
     *
     */
    public static void clearCache()
    {
        ClassMetamodelFactory.cache.clear();

    }




}
