/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.data;

import java.io.Serializable;
import java.util.Random;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class TestPayload implements Serializable
{
   private String name = null;
   private int id = -1;
   private TestPayload child = null;

   public static TestPayload createTestInstance()
   {
      TestPayload testObject = new TestPayload("foobar", new Random().nextInt());
      TestPayload child = new TestPayload("child", new Random().nextInt());
      testObject.setChild(child);
      return testObject;
   }

   public String toString()
   {
       return "TestPayLoad(" + name + "," + id + "," + child + ")";
   }

   /** Jbossserialization requires a default constructor.
    *  We could accept private constructors */
   public TestPayload()
   {
   }

   public TestPayload(String name, int id)
   {
      this.name = name;
      this.id = id;
   }

   public void setChild(TestPayload child)
   {
      this.child = child;
   }

   public TestPayload getChild()
   {
      return child;
   }

   public String getName()
   {
      return name;
   }

   public int getId()
   {
      return id;
   }

   public String getIdentity()
   {
      return name + id;
   }

   public boolean equals(Object obj)
   {
      if(obj == null)
      {
         return false;
      }
      if(obj instanceof TestPayload)
      {
         TestPayload tp = (TestPayload) obj;
         if(child == null)
         {
            if(tp.getChild() != null)
            {
               return false;
            }
            else
            {
               return getIdentity().equals(tp.getIdentity());
            }
         }
         else
         {
            if(tp.getChild() == null)
            {
               return false;
            }
            else
            {
               if(getChild().equals(tp.getChild()))
               {
                  return getIdentity().equals(tp.getIdentity());
               }
               else
               {
                  return false;
               }
            }
         }
      }
      else
      {
         return false;
      }
   }


}