/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.jboss.serial.data;

import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.Serializable;
import java.util.Random;

public class TestPrivateExternalizable implements Serializable {
	private static final long serialVersionUID = -5319420332895758592L;

	static Random random = new Random();

	private static class PrivateExternal implements Externalizable {
		public PrivateExternal()
		{
		}

		private static final long serialVersionUID = 7768980221765246464L;

		public int data1;

		public long data2;

		public void readExternal(ObjectInput in) throws IOException,
				ClassNotFoundException {
			data1 = in.readInt();
			data2 = in.readLong();
		}

		public void writeExternal(ObjectOutput out) throws IOException {
			out.writeInt(data1);
			out.writeLong(data2);
		}

		public int hashCode() {
			final int PRIME = 31;
			int result = 1;
			result = PRIME * result + data1;
			result = PRIME * result + (int) (data2 ^ (data2 >>> 32));
			return result;
		}

		public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null)
				return false;
			if (getClass() != obj.getClass())
				return false;
			final PrivateExternal other = (PrivateExternal) obj;
			if (data1 != other.data1)
				return false;
			if (data2 != other.data2)
				return false;
			return true;
		}
	}

	PrivateExternal external;

	public TestPrivateExternalizable() {
		external = new PrivateExternal();
		external.data1 = random.nextInt();
		external.data2 = random.nextLong();
	}

	public static TestPrivateExternalizable createTestInstance() {
		return new TestPrivateExternalizable();
	}

	public int hashCode() {

		return external.hashCode();
	}

	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final TestPrivateExternalizable other = (TestPrivateExternalizable) obj;
		if (external == null) {
			if (other.external != null)
				return false;
		} else if (!external.equals(other.external))
			return false;
		return true;
	}
}
