/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.jodatests;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import org.jboss.serial.io.JBossObjectInputStream;
import org.jboss.serial.io.JBossObjectOutputStream;
import org.jboss.serial.util.StringUtilBuffer;
import org.joda.time.chrono.ISOChronology;

import junit.framework.TestCase;

public class ISOCronologyTestCase extends TestCase
{
	
	public void testIsoCronologyOnJBoss() throws Exception
	{
		try
		{
			Object myTest = ISOChronology.getInstance();
	        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);
	
	        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
	        byteOut.reset();
	
	        JBossObjectOutputStream out = new JBossObjectOutputStream(byteOut,buffer);
	        out.writeObject(myTest);
	        out.flush();
	
	
	        ByteArrayInputStream byteInput = new ByteArrayInputStream(byteOut.toByteArray());
	        JBossObjectInputStream input = new JBossObjectInputStream(byteInput,buffer);
	
	        Object value = input.readObject();
	
	        assertSame(myTest.getClass(),value.getClass());
	        assertEquals(myTest,value);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			throw e;
		}

		
	}

	public void testIsoCronologyOnJava() throws Exception
	{
		try
		{
			Object myTest = ISOChronology.getInstance();
	        StringUtilBuffer buffer = new StringUtilBuffer(10*1024,10*1024);
	
	        ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
	        byteOut.reset();
	
	        ObjectOutputStream out = new ObjectOutputStream(byteOut);
	        out.writeObject(myTest);
	        out.flush();
	
	
	        ByteArrayInputStream byteInput = new ByteArrayInputStream(byteOut.toByteArray());
	        ObjectInputStream input = new ObjectInputStream(byteInput);
	
	        Object value = input.readObject();
	
	        //assertNotSame(myTest,value);
	        assertSame(myTest.getClass(),value.getClass());
	        assertEquals(myTest,value);
		}
		catch (Exception e)
		{
			e.printStackTrace();
			throw e;
		}

		
	}

}
