/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.nonserializable;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.jboss.serial.DataSerializationTest;
import org.jboss.serial.data.NonSerializableTest;
import org.jboss.serial.io.JBossObjectInputStream;
import org.jboss.serial.io.JBossObjectOutputStream;

/** This test will be executed for any Test* class under org.jboss.serial.data implementing NonSerializableTest */
public class ScanNonSerializableTestCase extends DataSerializationTest
{

	public void executTest(Object myTest) throws Throwable {
		if (!(myTest instanceof NonSerializableTest))
		{
			return;
		}
		System.out.println("Executing test for " + myTest.getClass().getName());
	    ByteArrayOutputStream byteOut = new ByteArrayOutputStream();
	
        byteOut.reset();

        JBossObjectOutputStream out = new JBossObjectOutputStream(byteOut);
        out.writeObject(myTest);
        out.flush();


        byte[] byteArray = byteOut.toByteArray();
        System.out.println(myTest.getClass().getName() +" produced " + byteArray.length + " on JBossSerialization");
        ByteArrayInputStream byteInput = new ByteArrayInputStream(byteArray);

        JBossObjectInputStream input = new JBossObjectInputStream(byteInput);

        Object value = input.readObject();

        assertNotSame(myTest,value);
        if (!(myTest instanceof String) && !(myTest instanceof String[]))
        {
            assertSame(myTest.getClass(),value.getClass());
            assertEquals(myTest,value);
        }
        
        if (value instanceof NonSerializableTest)
        {
        	((NonSerializableTest)value).doSomething();
        }

	}

	public void doTestJBossSerialization(Object myTest) throws Exception
    {

    }
	
}
