/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/

package org.jboss.serial.stream;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.OutputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import org.jboss.serial.DataSerializationTest;

/**
 * @author <a href="mailto:tom.elrod@jboss.com">Tom Elrod</a>
 */
public class StreamTestCase extends DataSerializationTest
{

   public static final int PORT = 5555;
   private ServerSocket serverSocket = null;
   private boolean keepRunning = true;
   private Object testObject = null;
   private static Thread serverThread = null;


   public void setUp() throws Exception
   {
      serverSocket = new ServerSocket(PORT, 100);
      runServer(serverSocket);
   }

   private void runServer(ServerSocket serverSocket)
   {
      final ServerSocket localRef = serverSocket;

      (serverThread=new Thread(new Runnable()
      {
         public void run()
         {
            while(keepRunning)
            {
               try
               {
                  Socket socket = localRef.accept();
                  BufferedInputStream bufStream = new BufferedInputStream(socket.getInputStream());
                  //ObjectInputStream objStream = new ObjectInputStream(bufStream);
                  //Object obj = objStream.readObject();



                  ObjectInput containerInput = StreamFactory.getObjectInputStream(bufStream);
                  Object objRead = containerInput.readObject();

                  System.out.println("Streamed object = " + objRead);
                  if (!(objRead instanceof Object[]))
                  {
                	  assertEquals(objRead, testObject);
                  }
               }
               catch(SocketException e)
               {

               }
               catch(Exception e)
               {
                  e.printStackTrace();
               }
            }
         }
      })).start();
   }

   public void executTest(Object dataObject) throws Exception
   {
      testObject = dataObject;

      Socket socket = new Socket("localhost", PORT);
      OutputStream output = socket.getOutputStream();

      ObjectOutput stream = StreamFactory.getObjectStream(output);
      stream.writeObject(testObject);

      stream.flush();
      stream.close();

      //ObjectOutputStream oos = new ObjectOutputStream(output);
      //oos.writeObject(testObject);
      //oos.flush();
      //oos.close();


      socket.close();

      // this is a bit of hack to allow the server to de-serialize the object and compare to the value referenced by testObject
      // otherwise, might reset the testObject with a new dataObject before is done deserializing and asserting.
      try
      {
         Thread.currentThread().sleep(100);
      }
      catch(InterruptedException e)
      {
      }
   }

   public void tearDown()
   {
      keepRunning = false;
      if(serverSocket != null && !serverSocket.isClosed())
      {
         try
         {
            serverSocket.close();
         }
         catch(IOException e)
         {
            e.printStackTrace();
         }
      }
      
      try
      {
    	  serverThread.interrupt();
      }
      catch (Throwable ignoredException)
      {
      }
   }

}